<?php

namespace EventPortal\Domain\Component;

use EventPortal\Domain\Field\Enums\FieldType;
use EventPortal\Domain\Util\Parameters;
use EventPortal\Enums\InertiaMethod;

class Button extends Component
{
    const TYPE = 'Button';

    /** @var InertiaMethod */
    public $method;

    /** @var string */
    public $route;

    /** @var Parameters */
    public $routeParameters;

    /** @var string */
    public $variant = 'primary';

    public function method(InertiaMethod $method): self
    {
        $this->method = $method->getValue();

        return $this;
    }

    public function route(string $route, ?array $parameters = []): self
    {
        $this->route = $route;

        if ($parameters) {
            $this->routeParameters = Parameters::make($parameters)->asPlain()->asArray();
        }

        return $this;
    }

    public function variant(string $variant): self
    {
        $this->variant = $variant;

        return $this;
    }

    public function type(): FieldType
    {
        return FieldType::Button();
    }

    /**
     * Represents the button in array format.
     */
    public function toArray(): array
    {
        $data = [
            'component' => self::TYPE,
            'label' => $this->label,
            'variant' => $this->variant,
        ];

        if ($this->method) {
            $data['method'] = $this->method;
        }

        if ($this->route) {
            $data['route'] = $this->route;
        }

        if ($this->routeParameters) {
            $data['routeParams'] = $this->routeParameters;
        }

        if ($this->classes) {
            $data['classes'] = $this->classes;
        }

        return $data;
    }
}
