<?php

namespace EventPortal\Domain\Component\Factory;

use EventPortal\Domain\Component\Display;
use EventPortal\Domain\Component\Dropdown\MultiSelect;
use EventPortal\Domain\Component\Dropdown\Select;
use EventPortal\Domain\Component\Input;
use EventPortal\Domain\Component\Input\PasswordInput;
use EventPortal\Domain\Component\Input\TextArea;
use EventPortal\Domain\Component\Input\TextInput;
use EventPortal\Domain\Component\Selection\Checkbox;
use EventPortal\Domain\Component\Selection\RadioButtons;
use EventPortal\Domain\Component\Selection\SlideToggle;
use EventPortal\Domain\Field\Models\Field;
use Illuminate\Support\Arr;

class InputFactory
{
    /** @var string<string> */
    public static $classes = [
        'TextInput' => TextInput::class,
        'Select' => Select::class,
        'MultiSelect' => MultiSelect::class,
        'PasswordInput' => PasswordInput::class,
        'TextArea' => TextArea::class,
        'Checkbox' => Checkbox::class,
        'RadioButtons' => RadioButtons::class,
        'SlideToggle' => SlideToggle::class,
    ];

    public function display(Field $field): Display
    {
        return Display::make($field->label)
            ->name($field->name)
            ->classes($field->data->get('classes'))
            ->when($field->response, static function ($field, $response): void {
                $field->value($response->data->get('value'));
            });
    }

    public function fromField(Field $field): Input
    {
        /** @var \EventPortal\Domain\Component\Input $fullPath */
        $fullPath = Arr::get(self::$classes, $field->type->getValue());

        $input = $fullPath::make($field->label)
            ->name($field->name)
            ->placeholder($field->data->get('placeholder'))
            ->clearable($field->data->get('clearable'))
            ->classes($field->data->get('classes'))
            ->validation($field->data->get('validation', null))
            ->createValidation($field->data->get('createValidation', null))
            ->updateValidation($field->data->get('updateValidation', null));

        if ($input->hasOptions() && $field->data->get('options')) {
            $input->options($field->data->get('options'));
        }

        $input->when($field->response, function ($field, $response): void {
            $field->value($response->data->get('value'));
        });

        return $input;
    }

    public function fromArray(array $data): Input
    {
        /** @var \EventPortal\Domain\Component\Input $fullPath */
        $fullPath = Arr::get(self::$classes, Arr::get($data, 'component'));

        $input = $fullPath::make(Arr::get($data, 'label'))
            ->name(Arr::get($data, 'name'))
            ->placeholder(Arr::get($data, 'placeholder'))
            ->clearable(Arr::get($data, 'clearable'))
            ->classes(Arr::get($data, 'classes'))
            ->validation(Arr::get($data, 'validation'))
            ->createValidation(Arr::get($data, 'createValidation'))
            ->updateValidation(Arr::get($data, 'updateValidation'));

        if (Arr::has($data, 'options')) {
            $input->options(Arr::get($data, 'options'));
        }

        $input->when(Arr::has($data, 'defaultValue'), static function ($input) use ($data): void {
            $input->value(Arr::get($data, 'defaultValue'));
        });

        return $input;
    }
}
