<?php

namespace EventPortal\Domain\Form\Factories;

use EventPortal\Contracts\HasFields;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Form\Facades\Section;
use EventPortal\Domain\Form\Form;
use EventPortal\Domain\Util\DefaultFields;

class FormFactory
{
    public function fromDefaultFields(DefaultFields $defaultFields, string $model): Form
    {
        $sections = $defaultFields
            ->asCollection()
            ->map(static function ($data) {
                return Section::make($data)->toArray();
            })
            ->toArray();

        return (new Form)->addSections($sections)->model($model);
    }

    public function fromModel(HasFields $model): Form
    {
        $model->loadMissing(['sections.children.response']);

        $sections = $model
            ->sections
            ->map(static function (Field $field) {
                return Section::make($field)->toArray();
            })
            ->toArray();

        return (new Form)
            ->model($model)
            ->addSections($sections);
    }

    public function display(HasFields $model): Form
    {
        $model->loadMissing(['sections.children.response']);

        $sections = $model
            ->sections
            ->map(static function ($field) {
                return Section::display($field)->toArray();
            })
            ->toArray();

        return (new Form())->addSections($sections);
    }
}
