<?php

namespace EventPortal\Providers;

use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Support\ServiceProvider;
use Inertia\Inertia;

class AppServiceProvider extends ServiceProvider
{
    /** @var string */
    protected $root = __DIR__.'/../..';

    public function boot(): void
    {
        $this->app->booted(function (): void {
            $this->loadRoutesFrom("{$this->root}/routes/routes.php");
        });

        $this->loadViewsFrom("{$this->root}/resources/views", 'EventPortal');
        
        $this->loadMigrationsFrom("{$this->root}/database/migrations");
        
        $this->loadFactoriesFrom("{$this->root}/database/factories");
        
        $this->publishes([
            "{$this->root}/config/defaults/event/fields.php" => config_path('/defaults/event/fields.php'),
            "{$this->root}/config/defaults/event/tableColumns.php" => config_path('/defaults/event/tableColumns.php'),
        ], 'defaults');

        $this->app->bind(\Illuminate\Contracts\Debug\ExceptionHandler::class, \EventPortal\Exceptions\Handler::class);

        Inertia::setRootView('EventPortal::app');

        Inertia::share([
            'errors' => function () {
                return session('errors') ? session('errors')->getBag('default')->getMessages() : (object) [];
            },
        ]);

        JsonResource::withoutWrapping();

        // EventPortalSDK Service Provider
        Form::make('event.create')
            ->registerSaveHandler(CreateEventModel::class);


        // Their Service Provider
        Form::make('event.create')
            ->addValidation(CheckThirdParty::class)
            ->registerSaveHandler(AssignToTeam::class)
            ->addFields(function () {
                return [
                    Section::make('Key Details')->fields([
                        Text::make('Name')->helpText('Event::name.help-text')->required(),
                        Select::make('Team')->options(function () {
                            return Team::query()->pluck('name', 'id');
                        }),
                    ]),
                ];
            });

        Form::make('event.edit')
            ->addValidation(CheckThirdParty::class)
            ->registerSaveHandler(AssignToTeam::class)
            ->addFields(function () {
                return [
                    Section::make('Key Details')->fields([
                        Text::make('Name')->helpText('Event::name.help-text')->required(),
                        Select::make('Team')->options(function (Event $event) {
                            return Team::query()->forEvent($event)->pluck('name', 'id');
                        })->value(function (Event $event) {
                            return $event->myteam->id;
                        }),
                    ]),
                ];
            });
    }
}
